@@ -1,6 +1,6 @@
 #!/usr/local/bin/python2.7
 
-## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
 ## Copyright (C) 2008 Novell, Inc.
 ## Author: Tim Waugh <twaugh@redhat.com>
 
@@ -596,7 +596,7 @@ class IPPAuthConnection(IPPConnection):
                                 host=host, port=port, encryption=encryption)
 
     def destroy (self):
-        del self.semantic
+        self.semantic = None
         IPPConnection.destroy (self)
 
     def _call_function (self, fn, *args, **kwds):
