@@ -1,7 +1,7 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
-## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010 Red Hat, Inc.
+## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -15,17 +15,18 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+import threading
 import cups
+import cupspk
+import gobject
 import gtk
+import os
+from errordialogs import *
 from debug import *
-
-_ = lambda x: x
+from gettext import gettext as _
 N_ = lambda x: x
-def set_gettext_function (fn):
-    global _
-    _ = fn
 
 class AuthDialog(gtk.Dialog):
     AUTH_FIELD={'username': N_("Username:"),
@@ -36,7 +37,8 @@ class AuthDialog(gtk.Dialog):
                   flags=gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
                   buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                            gtk.STOCK_OK, gtk.RESPONSE_OK),
-                  auth_info_required=['username', 'password']):
+                  auth_info_required=['username', 'password'],
+                  allow_remember=False):
         if title == None:
             title = _("Authentication")
         gtk.Dialog.__init__ (self, title, parent, flags, buttons)
@@ -75,6 +77,13 @@ class AuthDialog(gtk.Dialog):
         vbox.pack_start (table, False, False, 0)
         hbox.pack_start (vbox, False, False, 0)
         self.vbox.pack_start (hbox)
+
+        if allow_remember:
+            cb = gtk.CheckButton (_("Remember password"))
+            cb.set_active (False)
+            vbox.pack_start (cb)
+            self.remember_checkbox = cb
+
         self.vbox.show_all ()
 
     def set_prompt (self, prompt):
@@ -91,12 +100,42 @@ class AuthDialog(gtk.Dialog):
     def get_auth_info (self):
         return map (lambda x: x.get_text (), self.field_entry)
 
+    def get_remember_password (self):
+        try:
+            return self.remember_checkbox.get_active ()
+        except AttributeError:
+            return False
+
     def field_grab_focus (self, field):
         i = self.auth_info_required.index (field)
         self.field_entry[i].grab_focus ()
 
+###
+### An auth-info cache.
+###
+class _AuthInfoCache:
+    def __init__ (self):
+        self.creds = dict() # by (host,port)
+
+    def cache_auth_info (self, data, host=None, port=None):
+        if port == None:
+            port = 631
+
+        self.creds[(host,port)] = data
+
+    def lookup_auth_info (self, host=None, port=None):
+        if port == None:
+            port = 631
+
+        try:
+            return self.creds[(host,port)]
+        except KeyError:
+            return None
+
+global_authinfocache = _AuthInfoCache ()
+
 class Connection:
-    def __init__ (self, parent=None, try_as_root=True,
+    def __init__ (self, parent=None, try_as_root=True, lock=False,
                   host=None, port=None, encryption=None):
         if host != None:
             cups.setServer (host)
@@ -112,8 +151,26 @@ class Connection:
         self._server = cups.getServer ()
         self._port = cups.getPort()
         self._encryption = cups.getEncryption ()
-        self._connect ()
         self._prompt_allowed = True
+        self._operation_stack = []
+        self._lock = lock
+        self._gui_event = threading.Event ()
+
+        creds = global_authinfocache.lookup_auth_info (host=host, port=port)
+        if creds != None:
+            if (creds[0] != 'root' or try_as_root):
+                (self._use_user, self._use_password) = creds
+            del creds
+
+        self._connect ()
+
+    def _begin_operation (self, operation):
+        debugprint ("%s: Operation += %s" % (self, repr (operation)))
+        self._operation_stack.append (operation)
+
+    def _end_operation (self):
+        debugprint ("%s: Operation ended" % self)
+        self._operation_stack.pop ()
 
     def _get_prompt_allowed (self, ):
         return self._prompt_allowed
@@ -121,27 +178,36 @@ class Connection:
     def _set_prompt_allowed (self, allowed):
         self._prompt_allowed = allowed
 
-    def _connect (self):
+    def _set_lock (self, whether):
+        self._lock = whether
+
+    def _connect (self, allow_pk=True):
         cups.setUser (self._use_user)
-        try:
-            self._connection = cups.Connection (host=self._server,
-                                                port=self._port,
-                                                encryption=self._encryption)
-        except TypeError:
-            # Parameters for Connection() require pycups >= 1.9.40.
-            cups.setServer (self._server)
-            cups.setPort (self._port)
-            cups.setEncryption (self._encryption)
-            self._connection = cups.Connection ()
+
+        self._use_pk = (allow_pk and
+                        (self._server[0] == '/' or self._server == 'localhost')
+                        and os.getuid () != 0)
+        if self._use_pk:
+            create_object = cupspk.Connection
+        else:
+            create_object = cups.Connection
+
+        self._connection = create_object (host=self._server,
+                                            port=self._port,
+                                            encryption=self._encryption)
+
+        if self._use_pk:
+            self._connection.set_parent(self._parent)
 
         self._user = self._use_user
         debugprint ("Connected as user %s" % self._user)
-        methodtype = type (self._connection.getPrinters)
+        methodtype_lambda = type (self._connection.getPrinters)
+        methodtype_real = type (self._connection.addPrinter)
         for fname in dir (self._connection):
             if fname[0] == '_':
                 continue
             fn = getattr (self._connection, fname)
-            if type (fn) != methodtype:
+            if not type (fn) in [methodtype_lambda, methodtype_real]:
                 continue
             setattr (self, fname, self._make_binding (fname, fn))
 
@@ -151,15 +217,19 @@ class Connection:
     def _authloop (self, fname, fn, *args, **kwds):
         self._passes = 0
         c = self._connection
-        while self._perform_authentication () != 0:
-            if c != self._connection:
-                # We have reconnected.
-                fn = getattr (self._connection, fname)
-                c = self._connection
-
-            cups.setUser (self._use_user)
-
+        retry = False
+        while True:
             try:
+                if self._perform_authentication () == 0:
+                    break
+
+                if c != self._connection:
+                    # We have reconnected.
+                    fn = getattr (self._connection, fname)
+                    c = self._connection
+
+                cups.setUser (self._use_user)
+
                 result = fn.__call__ (*args, **kwds)
 
                 if fname == 'adminGetServerSettings':
@@ -169,23 +239,73 @@ class Connection:
                         raise cups.IPPError (cups.IPP_NOT_AUTHORIZED, '')
                 break
             except cups.IPPError, (e, m):
+                if self._use_pk and m == 'pkcancel':
+                    raise cups.IPPError (0, _("Operation canceled"))
                 if not self._cancel and (e == cups.IPP_NOT_AUTHORIZED or
                                          e == cups.IPP_FORBIDDEN):
                     self._failed (e == cups.IPP_FORBIDDEN)
+                elif not self._cancel and e == cups.IPP_SERVICE_UNAVAILABLE:
+                    if self._lock:
+                        self._gui_event.clear ()
+                        gobject.timeout_add (1, self._ask_retry_server_error, m)
+                        self._gui_event.wait ()
+                    else:
+                        self._ask_retry_server_error (m)
+
+                    if self._retry_response == gtk.RESPONSE_OK:
+                        debugprint ("retrying operation...")
+                        retry = True
+                        self._passes -= 1
+                        self._has_failed = True
+                    else:
+                        self._cancel = True
+                        raise
                 else:
                     if self._cancel and not self._cannot_auth:
                         raise cups.IPPError (0, _("Operation canceled"))
 
                     raise
             except cups.HTTPError, (s,):
-                if not self._cancel and (s == cups.HTTP_UNAUTHORIZED or
-                                         s == cups.HTTP_FORBIDDEN):
+                if not self._cancel:
                     self._failed (s == cups.HTTP_FORBIDDEN)
                 else:
                     raise
 
         return result
 
+    def _ask_retry_server_error (self, message):
+        if self._lock:
+            gtk.gdk.threads_enter ()
+
+        try:
+            msg = _("CUPS server error (%s)") % self._operation_stack[0]
+        except IndexError:
+            msg = _("CUPS server error")
+
+        d = gtk.MessageDialog (self._parent,
+                               gtk.DIALOG_MODAL |
+                               gtk.DIALOG_DESTROY_WITH_PARENT,
+                               gtk.MESSAGE_ERROR,
+                               gtk.BUTTONS_NONE,
+                               msg)
+                               
+        d.format_secondary_text (_("There was an error during the "
+                                   "CUPS operation: '%s'." % message))
+        d.add_buttons (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                       _("Retry"), gtk.RESPONSE_OK)
+        d.set_default_response (gtk.RESPONSE_OK)
+        if self._lock:
+            d.connect ("response", self._on_retry_server_error_response)
+            gtk.gdk.threads_leave ()
+        else:
+            self._retry_response = d.run ()
+            d.destroy ()
+
+    def _on_retry_server_error_response (self, dialog, response):
+        self._retry_response = response
+        dialog.destroy ()
+        self._gui_event.set ()
+
     def _failed (self, forbidden=False):
         self._has_failed = True
         self._forbidden = forbidden
@@ -242,7 +362,12 @@ class Connection:
                 debugprint ("Authentication: Try as root")
                 self._use_user = 'root'
                 self._auth_called = False
-                self._connect ()
+                try:
+                    self._connect (allow_pk=False)
+                except RuntimeError:
+                    raise cups.IPPError (cups.IPP_SERVICE_UNAVAILABLE,
+                                         'server-error-service-unavailable')
+
                 return 1
 
         if not self._prompt_allowed:
@@ -262,20 +387,75 @@ class Connection:
 
         # If we're previously prompted, explain why we're prompting again.
         if self._dialog_shown:
-            d = gtk.MessageDialog (self._parent,
-                                   gtk.DIALOG_MODAL |
-                                   gtk.DIALOG_DESTROY_WITH_PARENT,
-                                   gtk.MESSAGE_ERROR,
-                                   gtk.BUTTONS_CLOSE)
-            d.set_title (_("Not authorized"))
-            d.set_markup ('<span weight="bold" size="larger">' +
-                          _("Not authorized") + '</span>\n\n' +
-                          _("The password may be incorrect."))
+            if self._lock:
+                self._gui_event.clear ()
+                gobject.timeout_add (1, self._show_not_authorized_dialog)
+                self._gui_event.wait ()
+            else:
+                self._show_not_authorized_dialog ()
+
+        if self._lock:
+            self._gui_event.clear ()
+            gobject.timeout_add (1, self._perform_authentication_with_dialog)
+            self._gui_event.wait ()
+        else:
+            self._perform_authentication_with_dialog ()
+
+        if self._cancel:
+            debugprint ("cancelled")
+            return -1
+
+        cups.setUser (self._use_user)
+        debugprint ("Authentication: Reconnect")
+        try:
+            self._connect (allow_pk=False)
+        except RuntimeError:
+            raise cups.IPPError (cups.IPP_SERVICE_UNAVAILABLE,
+                                 'server-error-service-unavailable')
+
+        return 1
+
+    def _show_not_authorized_dialog (self):
+        if self._lock:
+            gtk.gdk.threads_enter ()
+        d = gtk.MessageDialog (self._parent,
+                               gtk.DIALOG_MODAL |
+                               gtk.DIALOG_DESTROY_WITH_PARENT,
+                               gtk.MESSAGE_ERROR,
+                               gtk.BUTTONS_CLOSE)
+        d.set_title (_("Not authorized"))
+        d.set_markup ('<span weight="bold" size="larger">' +
+                      _("Not authorized") + '</span>\n\n' +
+                      _("The password may be incorrect."))
+        if self._lock:
+            d.connect ("response", self._on_not_authorized_dialog_response)
+            d.show_all ()
+            d.show_now ()
+            gtk.gdk.threads_leave ()
+        else:
             d.run ()
             d.destroy ()
 
+    def _on_not_authorized_dialog_response (self, dialog, response):
+        self._gui_event.set ()
+        dialog.destroy ()
+
+    def _perform_authentication_with_dialog (self):
+        if self._lock:
+            gtk.gdk.threads_enter ()
+
         # Prompt.
-        d = AuthDialog (parent=self._parent)
+        if len (self._operation_stack) > 0:
+            try:
+                title = _("Authentication (%s)") % self._operation_stack[0]
+            except IndexError:
+                title = _("Authentication")
+
+            d = AuthDialog (title=title,
+                            parent=self._parent)
+        else:
+            d = AuthDialog (parent=self._parent)
+
         d.set_prompt (self._prompt)
         d.set_auth_info ([self._use_user, ''])
         d.field_grab_focus ('password')
@@ -283,24 +463,37 @@ class Connection:
         d.show_all ()
         d.show_now ()
         self._dialog_shown = True
-        response = d.run ()
+        if self._lock:
+            d.connect ("response", self._on_authentication_response)
+            gtk.gdk.threads_leave ()
+        else:
+            response = d.run ()
+            self._on_authentication_response (d, response)
+
+    def _on_authentication_response (self, dialog, response):
         (self._use_user,
-         self._use_password) = d.get_auth_info ()
-        d.destroy ()
+         self._use_password) = dialog.get_auth_info ()
+        global_authinfocache.cache_auth_info ((self._use_user,
+                                               self._use_password),
+                                              host=self._server,
+                                              port=self._port)
+        dialog.destroy ()
 
         if (response == gtk.RESPONSE_CANCEL or
             response == gtk.RESPONSE_DELETE_EVENT):
             self._cancel = True
-            return -1
 
-        cups.setUser (self._use_user)
-        debugprint ("Authentication: Reconnect")
-        self._connect ()
-
-        return 1
+        if self._lock:
+            self._gui_event.set ()
 
 if __name__ == '__main__':
     # Test it out.
+    gtk.gdk.threads_init ()
+    from timedops import TimedOperation
     set_debugging (True)
-    c = Connection (None)
-    print c.getFile ('/admin/conf/cupsd.conf', '/dev/stdout')
+    c = TimedOperation (Connection, args=(None,)).run ()
+    debugprint ("Connected")
+    c._set_lock (True)
+    print TimedOperation (c.getFile,
+                          args=('/admin/conf/cupsd.conf',
+                                '/dev/stdout')).run ()
