@@ -1,9 +1,10 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009 Red Hat, Inc.
-## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,18 +18,21 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
+import gobject
 import os
 import smburi
 import subprocess
+from timedops import TimedOperation, TimedSubprocess
 import urllib
 from base import *
 class CheckPrinterSanity(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Check printer sanity")
         troubleshooter.new_page (gtk.Label (), self)
+        self.troubleshooter = troubleshooter
 
     def display (self):
         # Collect information useful for the various checks.
@@ -41,22 +45,25 @@ class CheckPrinterSanity(Question):
 
         name = answers['cups_queue']
 
+        parent = self.troubleshooter.get_window ()
+
         # Find out if this is a printer or a class.
         try:
             cups.setServer ('')
-            c = cups.Connection ()
-            printers = c.getPrinters ()
+            c = TimedOperation (cups.Connection, parent=parent).run ()
+            printers = TimedOperation (c.getPrinters, parent=parent).run ()
             if printers.has_key (name):
                 self.answers['is_cups_class'] = False
                 queue = printers[name]
                 self.answers['cups_printer_dict'] = queue
             else:
                 self.answers['is_cups_class'] = True
-                classes = c.getClasses ()
+                classes = TimedOperation (c.getClasses, parent=parent).run ()
                 queue = classes[name]
                 self.answers['cups_class_dict'] = queue
 
-            attrs = c.getPrinterAttributes(name)
+            attrs = TimedOperation (c.getPrinterAttributes, (name,),
+                                    parent=parent).run ()
             self.answers['local_cups_queue_attributes'] = attrs
         except:
             pass
@@ -66,7 +73,7 @@ class CheckPrinterSanity(Question):
             uri = cups_printer_dict['device-uri']
             (scheme, rest) = urllib.splittype (uri)
             self.answers['cups_device_uri_scheme'] = scheme
-            if scheme in ["ipp", "http"]:
+            if scheme in ["ipp", "http", "https"]:
                 (hostport, rest) = urllib.splithost (rest)
                 (host, port) = urllib.splitnport (hostport, defport=631)
                 self.answers['remote_server_name'] = host
@@ -81,34 +88,40 @@ class CheckPrinterSanity(Question):
                 else:
                     cmdline = 'LC_ALL=C nmblookup "$HOST"'
                 try:
-                    p = subprocess.Popen (cmdline, shell=True,
-                                          stdin=file("/dev/null"),
-                                          stdout=subprocess.PIPE,
-                                          stderr=subprocess.PIPE)
-                    (stdout, stderr) = p.communicate ()
-                    self.answers['nmblookup_output'] = (stdout, stderr)
-                    for line in stdout.split ('\n'):
+                    p = TimedSubprocess (parent=parent,
+                                         timeout=5000,
+                                         args=cmdline, shell=True,
+                                         close_fds=True,
+                                         stdin=file("/dev/null"),
+                                         stdout=subprocess.PIPE,
+                                         stderr=subprocess.PIPE)
+                    result = p.run ()
+                    self.answers['nmblookup_output'] = result
+                    for line in result[0]:
                         if line.startswith ("querying"):
                             continue
                         spc = line.find (' ')
-                        if spc != -1:
+                        if (spc != -1 and
+                            not line[spc:].startswith (" failed ")):
+                            # Remember the IP address.
                             self.answers['remote_server_name'] = line[:spc]
                             break
-                except:
+                except OSError:
                     # Problem executing command.
                     pass
             elif scheme == "hp":
                 os.environ['URI'] = uri
                 try:
-                    p = subprocess.Popen ('LC_ALL=C DISPLAY= hp-info -d"$URI"',
-                                          shell=True,
-                                          stdin=file("/dev/null"),
-                                          stdout=subprocess.PIPE,
-                                          stderr=subprocess.PIPE)
-                    (stdout, stderr) = p.communicate ()
-                    self.answers['hplip_output'] = (stdout.split ('\n'),
-                                                    stderr.split ('\n'))
-                except:
+                    p = TimedSubprocess (parent=parent,
+                                         timeout=3000,
+                                         args='LC_ALL=C DISPLAY= hp-info -d"$URI"',
+                                         close_fds=True,
+                                         shell=True,
+                                         stdin=file("/dev/null"),
+                                         stdout=subprocess.PIPE,
+                                         stderr=subprocess.PIPE)
+                    self.answers['hplip_output'] = p.run ()
+                except OSError:
                     # Problem executing command.
                     pass
 
