@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.6
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,9 +17,10 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
+from timedops import TimedOperation
 from base import *
 class SchedulerNotRunning(Question):
     def __init__ (self, troubleshooter):
@@ -27,7 +28,7 @@ class SchedulerNotRunning(Question):
         page = self.initial_vbox (_("CUPS Service Stopped"),
                                   _("The CUPS print spooler does not appear "
                                     "to be running.  To correct this, run "
-                                    "'/usr/local/sbin/cupsd' "))
+                                    "'/etc/rc.d/cups start' "
                                     "as root in a terminal."))
         troubleshooter.new_page (page, self)
 
@@ -36,10 +37,14 @@ class SchedulerNotRunning(Question):
         if self.troubleshooter.answers.get ('cups_queue_listed', False):
             return False
 
+        parent = self.troubleshooter.get_window ()
+
         # Find out if CUPS is running.
         failure = False
         try:
-            c = cups.Connection ()
+            self.op = TimedOperation (cups.Connection,
+                                      parent=parent)
+            c = self.op.run ()
         except RuntimeError:
             failure = True
 
@@ -51,3 +56,6 @@ class SchedulerNotRunning(Question):
 
     def collect_answer (self):
         return self.answers
+
+    def cancel_operation (self):
+        self.op.cancel ()
